@@ -168,7 +168,6 @@ s_use (ignore)
 
   as_bad ("Unknown segment type");
   demand_empty_rest_of_line ();
-  return;
 }
 
 static void
@@ -176,7 +175,6 @@ s_data1 ()
 {
   subseg_set (SEG_DATA, 1);
   demand_empty_rest_of_line ();
-  return;
 }
 
 #endif /* OBJ_COFF */
@@ -346,12 +344,6 @@ md_begin ()
   define_some_regs ();
 }
 
-void
-md_end ()
-{
-  return;
-}
-
 /* Assemble a single instruction.  Its label has already been handled
    by the generic front end.  We just parse opcode and operands, and
    produce the bytes of data and relocation.  */
@@ -776,23 +768,7 @@ md_number_to_chars (buf, val, n)
      valueT val;
      int n;
 {
-
-  switch (n)
-    {
-
-    case 4:
-      *buf++ = val >> 24;
-      *buf++ = val >> 16;
-    case 2:
-      *buf++ = val >> 8;
-    case 1:
-      *buf = val;
-      break;
-
-    default:
-      as_fatal ("failed sanity check.");
-    }
-  return;
+  number_to_chars_bigendian (buf, val, n);
 }
 
 void
@@ -890,7 +866,6 @@ md_apply_fix (fixP, val)
       as_bad ("bad relocation type: 0x%02x", fixP->fx_r_type);
       break;
     }
-  return;
 }
 
 #ifdef OBJ_COFF
@@ -1016,7 +991,6 @@ print_insn (insn)
   fprintf (stderr, "\t\tX_add_number = %d\n",
 	   insn->exp.X_add_number);
   fprintf (stderr, "}\n");
-  return;
 }
 
 #endif
