@@ -41,52 +41,84 @@
  */
 
 struct fix
-  {
-    /* These small fields are grouped together for compactness of
-       this structure, and efficiency of access on some architectures.  */
-    char fx_pcrel_adjust;	/* pc-relative offset adjust */
-    unsigned fx_pcrel : 1;	/* TRUE: pc-relative. */
-    unsigned fx_im_disp : 1;	/* TRUE: value is a displacement */
-    unsigned fx_bsr : 1;	/* sequent-hack */
-    /* TRUE if target is a 'callj' (used by i960) */
-    unsigned fx_callj : 1;
-    short int fx_size;		/* How many bytes are involved? */
+{
+  /* These small fields are grouped together for compactness of
+     this structure, and efficiency of access on some architectures.  */
+
+  /* pc-relative offset adjust */
+  char fx_pcrel_adjust;
+
+  /* Is this a pc-relative relocation?  */
+  unsigned fx_pcrel : 1;
+
+  /* Is this value an immediate displacement?  */
+  unsigned fx_im_disp : 1;
+
+  /* This is a hack for the sequent support.
+     This probably can be folded into tc_fix_data, below.  */
+  unsigned fx_bsr : 1;
+
+  /* A bit for the CPU specific code.
+     This probably can be folded into tc_fix_data, below.  */
+  unsigned fx_tcbit : 1;
+
+  /* Has this relocation already been applied?  */
+  unsigned fx_done : 1;
+
+  /* How many bytes are involved? */
+  short int fx_size;
+
+  /* Which frag does this fix apply to?  */
+  fragS *fx_frag;
+
+  /* Where is the first byte to fix up?  */
+  long fx_where;
+
+  /* NULL or Symbol whose value we add in. */
+  symbolS *fx_addsy;
+
+  /* NULL or Symbol whose value we subtract. */
+  symbolS *fx_subsy;
+
+  /* Absolute number we add in. */
+  valueT fx_offset;
+
+  /* Next fixS in linked list, or NULL.  */
+  struct fix *fx_next;
+
+  /* If NULL, no bitfix's to do.  */
+  /* Only i960-coff and ns32k use this, and i960-coff stores an
+     integer.  This can probably be folded into tc_fix_data, below.  */
+  bit_fixS *fx_bit_fixP;
 
-    fragS *fx_frag;		/* Which frag? */
-    long fx_where;		/* Where is the 1st byte to fix up? */
-    symbolS *fx_addsy;		/* NULL or Symbol whose value we add in. */
-    symbolS *fx_subsy;		/* NULL or Symbol whose value we subtract. */
-    valueT fx_offset;		/* Absolute number we add in. */
-    struct fix *fx_next;	/* NULL or -> next fixS. */
-    bit_fixS *fx_bit_fixP;	/* IF NULL no bitfix's to do */
 #ifdef BFD_ASSEMBLER
-    bfd_reloc_code_real_type fx_r_type;
+  bfd_reloc_code_real_type fx_r_type;
 #else
 #ifdef NEED_FX_R_TYPE
-    /* Hack for machines where the type of reloc can't be
-       worked out by looking at how big it is.  */
-    int fx_r_type;
+  /* Hack for machines where the type of reloc can't be
+     worked out by looking at how big it is.  */
+  int fx_r_type;
 #endif
 #endif
 
-    /* This field is sort of misnamed.  It appears to be a sort of random
-       scratch field, for use by the back ends.  The main gas code doesn't
-       do anything but initialize it to zero.  The use of it does need to
-       be coordinated between the cpu and format files, though.  E.g., some
-       coff targets pass the `addend' field from the cpu file via this
-       field.  I don't know why the `fx_offset' field above can't be used
-       for that; investigate later and document. KR  */
-    valueT fx_addnumber;
+  /* This field is sort of misnamed.  It appears to be a sort of random
+     scratch field, for use by the back ends.  The main gas code doesn't
+     do anything but initialize it to zero.  The use of it does need to
+     be coordinated between the cpu and format files, though.  E.g., some
+     coff targets pass the `addend' field from the cpu file via this
+     field.  I don't know why the `fx_offset' field above can't be used
+     for that; investigate later and document. KR  */
+  valueT fx_addnumber;
 
-    /* The location of the instruction which created the reloc, used
-       in error messages.  */
-    char *fx_file;
-    unsigned fx_line;
+  /* The location of the instruction which created the reloc, used
+     in error messages.  */
+  char *fx_file;
+  unsigned fx_line;
 
-    /* Location where a backend can attach additional data
-       needed to perform fixups.  Used by both PA backends.  */
-    void *tc_fix_data;
-  };
+  /* Location where a backend can attach additional data
+     needed to perform fixups.  Used by both PA backends.  */
+  PTR tc_fix_data;
+};
 
 typedef struct fix fixS;
 
@@ -113,6 +145,9 @@ void record_alignment PARAMS ((segT seg, int align));
 void write_object_file PARAMS ((void));
 void relax_segment PARAMS ((struct frag * seg_frag_root, segT seg_type));
 
+void number_to_chars_littleendian PARAMS ((char *, valueT, int));
+void number_to_chars_bigendian    PARAMS ((char *, valueT, int));
+
 #ifdef BFD_ASSEMBLER
 fixS *fix_new PARAMS ((fragS * frag, int where, int size,
 		       symbolS * add_symbol, offsetT offset, int pcrel,
